library kplayer_platform_interface;

import 'dart:async';
/// a Player Statuses enum
enum PlayerEvent {
  /// on created.
  create,
  /// on initialized.
  init,
  /// on started.
  start,
  /// on restarted.
  restart,
  /// on playing.
  play,
  /// on replay.
  replay,
  /// on paused.
  pause,
  /// on toggle.
  toggle,
  /// on stopped.
  stop,
  /// on volume changed.
  volume,
  /// on speed changed.
  speed,
  /// on status changed.
  status,
  /// on position changed.
  position,
  /// on position changed, just in case idk
  duration,
  /// on dispose.
  dispose,
  /// on ended.
  end,
  /// unknown event. if you get this, probably something wrong
  unknown,
}

/// a Player Statuses enum
enum PlayerStatus {
  created,
  inited,
  ready,
  loading,
  buffering,
  playing,
  paused,
  stopped,
  ended,
  unknown,
}
/// a Player Media Types enum
enum PlayerMediaType {
  asset,
  network,
  file,
}

// Object handle Player media resources
class PlayerMedia {
  PlayerMedia(this.type, this.resource);

  final PlayerMediaType type;
  final String resource;

  factory PlayerMedia.asset(String _resource) {
    return PlayerMedia(PlayerMediaType.asset, _resource);
  }
  factory PlayerMedia.network(String _resource) {
    return PlayerMedia(PlayerMediaType.network, _resource);
  }
  factory PlayerMedia.file(String _resource) {
    return PlayerMedia(PlayerMediaType.file, _resource);
  }
}

/// the player interface
///
/// the interface apis
abstract class PlayerController {
  PlayerController({
    int? id,
    required this.media,
    this.autoPlay = false,
    this.once = false,
    this.loop = false,
  });

  // i know hhh
  // static bool _booted = false;
  // void needBoot(Function bootCallback) {
  //   if (!PlayerController._booted) {
  //     bootCallback();
  //   }
  // }

  static create(
      {int? id,
      required PlayerMedia media,
      bool? autoPlay,
      bool? once,
      bool? loop}) {}

  factory PlayerController.assets(media,
      {int? id, required bool autoPlay, bool? once}) {
    return PlayerController.create(
        id: id, media: PlayerMedia.asset(media), autoPlay: autoPlay, once: once)
      ..init();
  }

  factory PlayerController.network(media,
      {int? id, required bool autoPlay, bool? once}) {
    return PlayerController.create(
        id: id,
        media: PlayerMedia.network(media),
        autoPlay: autoPlay,
        once: once)
      ..init();
  }

  static void boot() {}

  final bool autoPlay;
  final bool once;
  final bool loop;
  final PlayerMedia media;

  static List<PlayerController> palyers = <PlayerController>[];
  void init();
  void replay();
  void play();
  void pause();
  void stop();
  @Deprecated("use the position setter")
  void seek(Duration position);
  void toggle();
  Function(PlayerEvent) callback = (PlayerEvent event){
    print("$event");
  };

  get package;
  bool get playing;
  Duration get position;
   set position(Duration duration);
  Duration get duration;
  double get volume;
   set volume(double volume);
  double get speed;
   set speed(double speed);
  PlayerStatus get status;
  set status(PlayerStatus status);

  @Deprecated('Use `streams.position` instead.')
  Stream<Duration> get positionStream;
  final PlayerStreamControllers _streamControllers = PlayerStreamControllers();
  PlayerStreams get streams => _streamControllers.streams;

  void dispose(){
    _streamControllers.dispose();
  }

  void notify(PlayerEvent event) {
    _streamControllers.status.add(status);
    _streamControllers.playing.add(playing);
    switch (event) {
      case PlayerEvent.position:
        _streamControllers.position.add(position);
        break;
      case PlayerEvent.duration:
        _streamControllers.duration.add(duration);
        break;
      case PlayerEvent.volume:
        _streamControllers.volume.add(volume);
        break;
      case PlayerEvent.speed:
        _streamControllers.speed.add(speed);
        break;
      default:
    }
    callback(event);
  }

}
/// All streams to for state managment
class PlayerStreams {
  final PlayerStreamControllers playerStreamControllers;
  PlayerStreams(this.playerStreamControllers);

  Stream<bool> get playing => playerStreamControllers.playing.stream;
  Stream<Duration> get position => playerStreamControllers.position.stream;
  Stream<Duration> get duration => playerStreamControllers.duration.stream;
  Stream<double> get volume => playerStreamControllers.volume.stream;
  Stream<double> get speed => playerStreamControllers.speed.stream;
  Stream<PlayerStatus> get status => playerStreamControllers.status.stream;

  void dispose() {
    playerStreamControllers.dispose();
  }

}
class PlayerStreamControllers {
  
  final StreamController<bool> playing = StreamController.broadcast();
  final StreamController<Duration> position = StreamController.broadcast();
  final StreamController<Duration> duration = StreamController.broadcast();
  final StreamController<double> volume = StreamController.broadcast();
  final StreamController<double> speed = StreamController.broadcast();
  final StreamController<PlayerStatus> status = StreamController.broadcast();

  get streams => PlayerStreams(this);

  void dispose() {
    playing.close();
    position.close();
    duration.close();
    volume.close();
    speed.close();
    status.close();
  }

}

class PlayerValue {
  final bool playing;
  final Duration position;
  final Duration duration;
  final double volume;
  final double speed;
  final PlayerStatus status;

  PlayerValue(
      {required this.playing,
      required this.position,
      required this.duration,
      required this.volume,
      required this.speed,
      required this.status});
}
